library devtools;

import 'dart:async';
import 'dart:io';

import 'package:dartz/dartz.dart';
import 'package:http/http.dart' as http;

import 'failure.dart';

const TIMEOUT_EXCEPTION_MESSAGE =
    'The connection has timed out, Please try again!';

enum Method {
  GET,
  POST,
  PUT,
  DELETE,
}

class Api {
  String baseUrl = "";
  Map<String, String> headers = {};
  Duration timeout = Duration(seconds: 20);
  late http.Response _response;
  HttpClient httpClient = HttpClient();

  Future<Either> request(
    Method method,
    String route, {
    Map<String, String> params = const {},
  }) async {
    Uri uri = _buildUri(route, params);
    try {
      await _makeRequest(method, uri);

      if (_response.statusCode == 200) {
        return Right(_response);
      } else {
        return Left(
          Failure(
            _response.statusCode,
            _response.body,
          ),
        );
      }
    } catch (e) {
      return Left(
        Failure(
          _response.statusCode,
          e.toString(),
        ),
      );
    }
  }

  Stream<Either> requestAsStream(
    Method method,
    String route, {
    Map<String, String> params = const {},
  }) async* {
    Uri uri = _buildUri(route, params);
    http.Response? response;
    try {
      await _makeRequest(method, uri);
      if (_response.statusCode == 200) {
        yield Right(response);
      }
    } catch (e) {
      yield Left(
        Failure(
          _response.statusCode,
          e.toString(),
        ),
      );
    }
  }

  Future<void> _makeRequest(Method method, Uri uri) async {
    switch (method) {
      case Method.GET:
        _response = await http.get(uri, headers: headers).timeout(timeout,
            onTimeout: () {
          throw TimeoutException(TIMEOUT_EXCEPTION_MESSAGE);
        });
        break;
      case Method.POST:
        _response = await http.post(uri, headers: headers).timeout(timeout,
            onTimeout: () {
          throw TimeoutException(TIMEOUT_EXCEPTION_MESSAGE);
        });
        break;
      case Method.PUT:
        _response = await http.put(uri, headers: headers).timeout(timeout,
            onTimeout: () {
          throw TimeoutException(TIMEOUT_EXCEPTION_MESSAGE);
        });
        break;
      case Method.DELETE:
        _response = await http.delete(uri, headers: headers).timeout(timeout,
            onTimeout: () {
          throw TimeoutException(TIMEOUT_EXCEPTION_MESSAGE);
        });
        break;
    }
  }

  String _paramStringFromMap(Map<String, dynamic> params) {
    String paramString = "";

    params.forEach(
      (key, value) {
        paramString += key + "=" + value + "&";
      },
    );

    return paramString;
  }

  Uri _buildUri(String route, Map<String, String> params) {
    Uri uri = Uri.parse(baseUrl + "/" + route);

    if (params.length != 0) {
      uri = Uri.parse(
        baseUrl + "/" + route + "?" + _paramStringFromMap(params),
      );
    }
    return uri;
  }
}
